"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const DEFAULT_MAX_CHAR_LENGTH = 255;
const DEFAULT_MIN_DATE = '01-01-1970';
exports.dummyCustomSchema = {
    maxCharLength: DEFAULT_MAX_CHAR_LENGTH,
    minDate: DEFAULT_MIN_DATE,
    ignoredTables: [],
    tables: [],
    values: {}
};
class Analyser {
    constructor(dbConnector, customSchema) {
        this.dbConnector = dbConnector;
        this.customSchema = customSchema;
        this.extractForeignKeys = (table) => __awaiter(this, void 0, void 0, function* () {
            const foreignKeys = yield this.dbConnector.getForeignKeys(table);
            const customTable = this.customSchema.tables.find(t => t.name.toLowerCase() === table.name.toLowerCase()) || {
                name: '',
                columns: [],
                lines: 0,
            };
            for (let c = 0; c < table.columns.length; c++) {
                const column = table.columns[c];
                const customColumn = customTable.columns.find(cc => cc.name.toLowerCase() === column.name.toLowerCase());
                const match = foreignKeys.find((fk) => fk.column.toLowerCase() === column.name.toLowerCase());
                if (match) {
                    column.foreignKey = { table: match.foreignTable, column: match.foreignColumn };
                    column.options.unique = column.options.unique || match.uniqueIndex;
                }
                if (customColumn) {
                    column.options = Object.assign({}, column.options, customColumn.options);
                    column.foreignKey = customColumn.foreignKey;
                    column.values = customColumn.values;
                }
                if (column.foreignKey) {
                    table.referencedTables.push(column.foreignKey.table);
                }
            }
        });
    }
    analyse() {
        return __awaiter(this, void 0, void 0, function* () {
            let tables = yield this.dbConnector.getTablesInformation(this.customSchema.ignoredTables);
            tables = yield Promise.all(tables.map((table) => __awaiter(this, void 0, void 0, function* () {
                yield this.customizeTable(table);
                yield this.extractColumns(table);
                yield this.extractForeignKeys(table);
                return table;
            })));
            return this.generateJson(this.orderTablesByForeignKeys(tables));
        });
    }
    customizeTable(table) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.customSchema.tables) {
                const customTable = this.customSchema.tables.find(t => t.name && t.name.toLowerCase() === table.name.toLowerCase());
                if (customTable) {
                    table.lines = customTable.lines;
                    table.before = customTable.before;
                    table.after = customTable.after;
                }
            }
        });
    }
    extractColumns(table) {
        return __awaiter(this, void 0, void 0, function* () {
            const columns = yield this.dbConnector.getColumnsInformation(table);
            columns
                .filter((column) => {
                return ['enum', 'set'].includes(column.DATA_TYPE || '');
            }).forEach((column) => {
                column.NUMERIC_PRECISION = column.COLUMN_TYPE.match(/[enum,set]\((.*)\)$/)[1].split('\',\'').length;
            });
            table.columns = columns.map((column) => {
                const options = {
                    max: 0,
                    min: 0,
                    autoIncrement: false,
                    nullable: false,
                    unique: false,
                    unsigned: false
                };
                if (column.COLUMN_KEY.match(/PRI|UNI/ig))
                    options.unique = true;
                if (column.IS_NULLABLE === 'YES')
                    options.nullable = true;
                options.max = column.CHARACTER_MAXIMUM_LENGTH || column.NUMERIC_PRECISION;
                if (column.COLUMN_TYPE.includes('unsigned'))
                    options.unsigned = true;
                if (column.EXTRA.includes('auto_increment'))
                    options.autoIncrement = true;
                if (['timestamp', 'datetime', 'date'].includes(column.DATA_TYPE))
                    options.minDate = this.customSchema.minDate;
                switch (column.DATA_TYPE) {
                    case 'bit':
                        break;
                    case 'tinyint':
                        if (options.unsigned) {
                            options.min = 0;
                            options.max = 255;
                        }
                        else {
                            options.min = -128;
                            options.max = 127;
                        }
                        break;
                    case 'bool':
                    case 'boolean':
                        break;
                    case 'smallint':
                        if (options.unsigned) {
                            options.min = 0;
                            options.max = 65535;
                        }
                        else {
                            options.min = -32768;
                            options.max = 32767;
                        }
                        break;
                    case 'mediumint':
                        if (options.unsigned) {
                            options.min = 0;
                            options.max = 16777215;
                        }
                        else {
                            options.min = -8388608;
                            options.max = 8388607;
                        }
                        break;
                    case 'int':
                    case 'integer':
                    case 'bigint':
                        if (options.unsigned) {
                            options.min = 0;
                            options.max = 2147483647;
                        }
                        else {
                            options.min = -2147483648;
                            options.max = 2147483647;
                        }
                        break;
                    case 'decimal':
                    case 'dec':
                    case 'float':
                    case 'double':
                        if (options.unsigned) {
                            options.min = 0;
                            options.max = 2147483647;
                        }
                        else {
                            options.min = -2147483648;
                            options.max = 2147483647;
                        }
                        break;
                    case 'date':
                    case 'datetime':
                    case 'timestamp':
                        options.minDate = this.customSchema.minDate || '01-01-1970';
                        options.maxDate = undefined;
                        break;
                    case 'time':
                        break;
                    case 'year':
                        options.min = parseInt(this.customSchema.minDate.substring(6)) || 1901;
                        options.max = 2155;
                        break;
                    case 'varchar':
                    case 'char':
                    case 'binary':
                    case 'varbinary':
                        break;
                    case 'tinyblob':
                        break;
                    case 'text':
                    case 'mediumtext':
                    case 'longtext':
                        break;
                    case 'blob':
                    case 'mediumblob': // 16777215
                    case 'longblob': // 4,294,967,295
                        break;
                    case 'set':
                        break;
                    case 'enum':
                        break;
                }
                return {
                    name: column.COLUMN_NAME,
                    generator: column.DATA_TYPE,
                    options,
                };
            });
        });
    }
    orderTablesByForeignKeys(tables) {
        let sortedTables = [];
        const recursive = (branch) => {
            const table = branch[branch.length - 1];
            while (table.referencedTables.length > 0) {
                const tableName = table.referencedTables.pop();
                const referencedTable = tables.find((t) => {
                    return t.name === tableName;
                });
                if (referencedTable)
                    recursive([].concat(branch, referencedTable));
            }
            ;
            if (table.referencedTables.length === 0) {
                if (sortedTables.find((t) => t.name.toLowerCase() === table.name.toLowerCase()))
                    return;
                sortedTables.push({
                    name: table.name,
                    lines: table.lines,
                    columns: table.columns,
                    before: table.before,
                    after: table.after,
                });
                branch.pop();
                return;
            }
        };
        tables.forEach((table) => {
            recursive([table]);
        });
        return sortedTables;
    }
    ;
    generateJson(tables) {
        return {
            maxCharLength: this.customSchema.maxCharLength || DEFAULT_MAX_CHAR_LENGTH,
            minDate: this.customSchema.minDate || DEFAULT_MIN_DATE,
            ignoredTables: this.customSchema.ignoredTables,
            tables: tables,
            values: this.customSchema.values,
        };
    }
}
exports.Analyser = Analyser;
